{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {
    "collapsed": false
   },
   "source": [
    "**Example, i.e., not great, strategy.** The elevator starts on the ground floor and moves all the way to the top floor, stopping at every floor in between. When it reaches the top floor, the elevator changes direction and moves all the way back down to the ground floor, again stopping at every floor in between. At every floor where the elevator stops any passengers who want to get off at that floor leave, and any passengers who want to get on enter, as long as there is space in the elevator. If the elevator is full, passengers on that floor have to wait. Upon reaching the ground floor, the elevator repeats the cycle of moving all the way up and all the way down the building."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 18,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
    "import numpy as np\n",
    "import random"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 19,
   "metadata": {
    "cocalc": {
     "outputs": {
      "0": {
       "name": "input",
       "opts": {
        "password": false,
        "prompt": "How many passengers are there?: "
       },
       "output_type": "stream",
       "value": "4"
      },
      "1": {
       "name": "input",
       "opts": {
        "password": false,
        "prompt": "How many floors are there?: "
       },
       "output_type": "stream",
       "value": "4"
      }
     }
    },
    "collapsed": false
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": "How many passengers are there?:  4"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": "How many floors are there?:  4"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Elevator picks up passenger 3 at floor 0\n",
      "Elevator moves from floor 0 to floor 1\n",
      "Elevator moves from floor 1 to floor 2\n",
      "Elevator picks up passenger 1 at floor 2\n",
      "Elevator drops off passenger 1 at floor 2\n",
      "Elevator drops off passenger 3 at floor 2\n",
      "Elevator moves from floor 2 to floor 3\n",
      "Elevator picks up passenger 2 at floor 3\n",
      "Elevator drops off passenger 2 at floor 3\n",
      "Elevator moves from floor 3 to floor 2\n",
      "Elevator moves from floor 2 to floor 1\n",
      "Elevator moves from floor 1 to floor 0\n",
      "Elevator moves from floor 0 to floor 1\n",
      "Elevator moves from floor 1 to floor 2\n",
      "Elevator moves from floor 2 to floor 3\n",
      "Elevator picks up passenger 4 at floor 3\n",
      "Elevator moves from floor 3 to floor 2\n",
      "Elevator moves from floor 2 to floor 1\n",
      "Elevator drops off passenger 4 at floor 1\n",
      "All passengers have been served - End of simulation\n"
     ]
    }
   ],
   "source": [
    "class Elevator():\n",
    "    \"\"\"\n",
    "    Elevator class ..\n",
    "    \"\"\"\n",
    "    def __init__(self, num_floors, requests, curr_floor=0, up=True, capacity=5, num_passengers=0):\n",
    "        self.curr_floor = curr_floor #the floor that the elevator is currently on\n",
    "        self.requests = requests #list of requests from all the passengers, this is specified at once in the start by user\n",
    "        self.up = up #indicator whether the elevator is moving up or down\n",
    "        self.capacity = capacity #maximum number of passengers that the elevator could carry\n",
    "        self.num_floors = num_floors #number of floors in the building\n",
    "        self.num_passengers = num_passengers #number of passengers currently in the elevator\n",
    "\n",
    "    def add_request(self, passenger): #add a request into the list of requests\n",
    "        self.requests.append(passenger) #this is not used in the simulation because we assume all requests were added at once before the simulation started\n",
    "\n",
    "    def remove_request(self, passenger): #remove a request from the list of requests\n",
    "        self.requests.remove(passenger)\n",
    "\n",
    "    def move(self): #the elevator moves by one floor, either up or down\n",
    "        #The elevator moves up if it is in the base floor\n",
    "        if self.curr_floor == 0:\n",
    "            self.up = True\n",
    "        #The elevator changes direction if it is in the top floor\n",
    "        if self.curr_floor == self.num_floors-1:\n",
    "            self.up = False\n",
    "        #If the direction is up, it continues to move up\n",
    "        if self.up == True:\n",
    "            self.curr_floor += 1\n",
    "        #If the direction is down, it continues to move down\n",
    "        if self.up == False:\n",
    "            self.curr_floor -= 1\n",
    "\n",
    "class Passenger():\n",
    "    \"\"\"\n",
    "    Passenger class ..\n",
    "    \"\"\"\n",
    "    def __init__(self, index, num_floors, current=0, destination=0, in_elevator=False, served=False):\n",
    "        self.current=np.random.randint(0,num_floors) #the floor which the passenger comes from\n",
    "        self.destination=np.random.randint(0,num_floors) #the floor which the passenger wants to go to\n",
    "        self.index=index #index of the passenger from 1 to total number of passengers\n",
    "        self.in_elevator=in_elevator #indicator whether the passenger is currently in the elevator or waiting outside\n",
    "        #self.served=served #indicator whether the passenger is already delivered to the destination. this is not used for this particular strategy.\n",
    "\n",
    "class Building():\n",
    "    \"\"\"\n",
    "    Building class ..\n",
    "    \"\"\"\n",
    "    def __init__(self, num_floors, passenger_list):\n",
    "        self.num_floors = num_floors #number of floors in the building, floor will be indexed from 0 to num_floors-1\n",
    "        self.passenger_list = passenger_list #list of passengers\n",
    "\n",
    "def main():\n",
    "    try:\n",
    "        #User inputs number of passengers and number of floors\n",
    "        total_passengers = int(input(\"How many passengers are there?: \"))\n",
    "        total_floors = int(input(\"How many floors are there?: \"))\n",
    "        #Generate a list of passengers based on total number of passengers\n",
    "        #This would also be the list of requests as we assume each passenger uses the elevator only once.\n",
    "        list_of_passengers = [Passenger(i, total_floors) for i in range(1,total_passengers+1)]\n",
    "        #If you want to see the current and destination of passengers, uncomment print(passengers)\n",
    "        passengers=[]\n",
    "        for p in list_of_passengers:\n",
    "            passengers.append((p.current,p.destination))\n",
    "        #print (passengers)\n",
    "        #Generate an object \"elevator\"\n",
    "        elevator = Elevator(total_floors, list_of_passengers)\n",
    "        #Generate an object \"building\"\n",
    "        building = Building(total_floors, list_of_passengers)\n",
    "        #The elevator keeps moving until we have no request left in our list of requests.\n",
    "        while len(elevator.requests) > 0:\n",
    "            for passenger in elevator.requests:\n",
    "                # Pick-up:\n",
    "                # if the passenger is on the floor that the elevator stops, is outside the elevator, and\n",
    "                # the number of passengers in the elevator is smaller than its capacity, the passenger is picked up.\n",
    "                if passenger.current == elevator.curr_floor \\\n",
    "                and passenger.in_elevator == False \\\n",
    "                and elevator.num_passengers < elevator.capacity:\n",
    "                    passenger.in_elevator = True\n",
    "                    elevator.num_passengers+=1\n",
    "                    print(\"Elevator picks up passenger %s at floor %s\" % \n",
    "                          (passenger.index, elevator.curr_floor))\n",
    "                # Drop-off:\n",
    "                # if the passenger destination is on the floor that the elevator stops\n",
    "                #the passenger is inside the elevator, the passenger is dropped off.\n",
    "                if passenger.destination == elevator.curr_floor \\\n",
    "                and passenger.in_elevator == True:\n",
    "                    elevator.remove_request(passenger)\n",
    "                    elevator.num_passengers-=1\n",
    "                    print (\"Elevator drops off passenger %s at floor %s\" % \n",
    "                           (passenger.index, elevator.curr_floor))\n",
    "            #if there is no request left in the list of requests, we end the simulation and break the loop.\n",
    "            if len(elevator.requests)==0:\n",
    "                print (\"All passengers have been served - End of simulation\")\n",
    "                break\n",
    "            elevator.move()\n",
    "            if elevator.up == True:\n",
    "                print (\"Elevator moves from floor %s to floor %s\" % (elevator.curr_floor-1,elevator.curr_floor))\n",
    "            elif elevator.up == False:\n",
    "                print (\"Elevator moves from floor %s to floor %s\" % (elevator.curr_floor+1,elevator.curr_floor))\n",
    "    except ValueError:\n",
    "        print (\"Invalid input, please try again\")\n",
    "        main()\n",
    "if __name__ == \"__main__\":\n",
    "    main()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 0,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (old Anaconda 3)",
   "language": "python",
   "name": "anaconda3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.5.5"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 0
}